@@ -29,6 +29,8 @@
 TAILQ_HEAD(interfaces, lldpd_interface);
 #ifdef ENABLE_DOT1
 TAILQ_HEAD(vlans, lldpd_vlan);
+TAILQ_HEAD(ppvids, lldpd_ppvid);
+TAILQ_HEAD(pids, lldpd_pi);
 #endif
 
 #define ntohll(x) (((u_int64_t)(ntohl((int)((x << 32) >> 32))) << 32) |	\
@@ -393,6 +395,56 @@ get_vlans(int s, struct vlans *vls, char *interface, int nb)
 		fatalx("get_vlans: unable to retrieve the list of vlans");
 	return 1;
 }
+
+static int
+get_ppvids(int s, struct ppvids *pvs, char *interface, int nb)
+{
+	void *p;
+	struct hmsg *h;
+
+	if ((h = (struct hmsg *)malloc(MAX_HMSGSIZE)) == NULL)
+		fatal(NULL);
+	ctl_msg_init(h, HMSG_GET_PPVIDS);
+	strlcpy((char *)&h->data, interface, IFNAMSIZ);
+	memcpy((char*)&h->data + IFNAMSIZ, &nb, sizeof(int));
+	h->hdr.len += IFNAMSIZ + sizeof(int);
+	if (ctl_msg_send(s, h) == -1)
+		fatalx("get_ppvids: unable to send request");
+	if (ctl_msg_recv(s, h) == -1)
+		fatalx("get_ppvids: unable to receive answer");
+	if (h->hdr.type != HMSG_GET_PPVIDS)
+		fatalx("get_ppvids: unknown answer type received");
+	p = &h->data;
+	if (ctl_msg_unpack_list(STRUCT_LLDPD_PPVID,
+		pvs, sizeof(struct lldpd_ppvid), h, &p) == -1)
+		fatalx("get_ppvids: unable to retrieve the list of ppvids");
+	return 1;
+}
+
+static int
+get_pids(int s, struct pids *pids, char *interface, int nb)
+{
+	void *p;
+	struct hmsg *h;
+
+	if ((h = (struct hmsg *)malloc(MAX_HMSGSIZE)) == NULL)
+		fatal(NULL);
+	ctl_msg_init(h, HMSG_GET_PIDS);
+	strlcpy((char *)&h->data, interface, IFNAMSIZ);
+	memcpy((char*)&h->data + IFNAMSIZ, &nb, sizeof(int));
+	h->hdr.len += IFNAMSIZ + sizeof(int);
+	if (ctl_msg_send(s, h) == -1)
+		fatalx("get_pids: unable to send request");
+	if (ctl_msg_recv(s, h) == -1)
+		fatalx("get_pids: unable to receive answer");
+	if (h->hdr.type != HMSG_GET_PIDS)
+		fatalx("get_pids: unknown answer type received");
+	p = &h->data;
+	if (ctl_msg_unpack_list(STRUCT_LLDPD_PI,
+		pids, sizeof(struct lldpd_pi), h, &p) == -1)
+		fatalx("get_pids: unable to retrieve the list of pids");
+	return 1;
+}
 #endif
 
 static int
@@ -988,7 +1040,7 @@ display_vlans(struct writer *w, struct lldpd_port *port)
 
 		tag_start(w, "vlan", "VLAN");
 		tag_attr(w, "vlan-id", "", u2str(vlan->v_vid));
-		if (foundpvid)
+		if (port->p_pvid == vlan->v_vid)
 			tag_attr(w, "pvid", "pvid", "yes");
 		tag_data(w, vlan->v_name);
 		tag_end(w);
@@ -1000,6 +1052,41 @@ display_vlans(struct writer *w, struct lldpd_port *port)
 		tag_end(w);
 	}
 }
+
+static void
+display_ppvids(struct writer *w, struct lldpd_port *port)
+{
+	struct lldpd_ppvid *ppvid;
+	TAILQ_FOREACH(ppvid, &port->p_ppvids, p_entries) {
+		tag_start(w, "ppvid", "PPVID");
+		if (ppvid->p_ppvid)
+			tag_attr(w, "value", "", u2str(ppvid->p_ppvid));
+		tag_attr(w, "supported", "supported",
+			 (ppvid->p_cap_status & LLDPD_PPVID_CAP_SUPPORTED)?"yes":"no");
+		tag_attr(w, "enabled", "enabled",
+			 (ppvid->p_cap_status & LLDPD_PPVID_CAP_ENABLED)?"yes":"no");
+		tag_end(w);
+	}
+}
+
+static void
+display_pids(struct writer *w, struct lldpd_port *port)
+{
+	struct lldpd_pi *pi;
+	char *hex;
+	TAILQ_FOREACH(pi, &port->p_pids, p_entries) {
+		if (!pi->p_pi_len) continue;
+		tag_start(w, "pi", "PI");
+		/* Convert to hex for display */
+		if ((hex = malloc(pi->p_pi_len * 2 + 1)) == NULL)
+			fatal(NULL);
+		for (int i = 0; i < pi->p_pi_len; i++)
+			snprintf(hex + 2*i, 3, "%02X", (unsigned char)pi->p_pi[i]);
+		tag_data(w, hex);
+		tag_end(w);
+		free(hex);
+	}
+}
 #endif
 
 static const char*
@@ -1027,6 +1114,8 @@ display_interfaces(int s, const char * fmt, int argc, char *argv[])
 	struct interfaces ifs;
 #ifdef ENABLE_DOT1
 	struct vlans vls;
+	struct ppvids pvs;
+	struct pids pids;
 #endif
 	struct lldpd_interface *iff;
 	struct lldpd_chassis chassis;
@@ -1080,6 +1169,16 @@ display_interfaces(int s, const char * fmt, int argc, char *argv[])
 			if (!TAILQ_EMPTY(&port.p_vlans) || port.p_pvid) {
 				display_vlans(w, &port);
 			}
+			if (get_ppvids(s, &pvs, iff->name, i) != -1)
+				memcpy(&port.p_ppvids, &pvs, sizeof(struct ppvids));
+			if (!TAILQ_EMPTY(&port.p_ppvids)) {
+				display_ppvids(w, &port);
+			}
+			if (get_pids(s, &pids, iff->name, i) != -1)
+				memcpy(&port.p_pids, &pids, sizeof(struct pids));
+			if (!TAILQ_EMPTY(&port.p_pids)) {
+				display_pids(w, &port);
+			}
 #endif
 #ifdef ENABLE_LLDPMED
 			if (port.p_med_cap_enabled) {
