@@ -115,6 +115,7 @@ usage(void)
 	fprintf(stderr, "-S descr Override the default system description.\n");
 	fprintf(stderr, "-m IP    Specify the management address of this system.\n");
 	fprintf(stderr, "-H mode  Specify the behaviour when detecting multiple neighbors.\n");
+	fprintf(stderr, "-I iface Limit interfaces to use.\n");
 #ifdef ENABLE_LLDPMED
 	fprintf(stderr, "-M class Enable emission of LLDP-MED frame. 'class' should be one of:\n");
 	fprintf(stderr, "             1 Generic Endpoint (Class I)\n");
@@ -185,6 +186,8 @@ lldpd_alloc_hardware(struct lldpd *cfg, char *name)
 #endif
 #ifdef ENABLE_DOT1
 	TAILQ_INIT(&hardware->h_lport.p_vlans);
+	TAILQ_INIT(&hardware->h_lport.p_ppvids);
+	TAILQ_INIT(&hardware->h_lport.p_pids);
 #endif
 	return hardware;
 }
@@ -203,6 +206,33 @@ lldpd_vlan_cleanup(struct lldpd_port *port)
 		free(vlan);
 	}
 }
+
+void
+lldpd_ppvid_cleanup(struct lldpd_port *port)
+{
+	struct lldpd_ppvid *ppvid, *ppvid_next;
+	for (ppvid = TAILQ_FIRST(&port->p_ppvids);
+	    ppvid != NULL;
+	    ppvid = ppvid_next) {
+		ppvid_next = TAILQ_NEXT(ppvid, p_entries);
+		TAILQ_REMOVE(&port->p_ppvids, ppvid, p_entries);
+		free(ppvid);
+	}
+}
+
+void
+lldpd_pi_cleanup(struct lldpd_port *port)
+{
+	struct lldpd_pi *pi, *pi_next;
+	for (pi = TAILQ_FIRST(&port->p_pids);
+	    pi != NULL;
+	    pi = pi_next) {
+		free(pi->p_pi);
+		pi_next = TAILQ_NEXT(pi, p_entries);
+		TAILQ_REMOVE(&port->p_pids, pi, p_entries);
+		free(pi);
+	}
+}
 #endif
 
 /* If `all' is true, clear all information, including information that
@@ -218,6 +248,8 @@ lldpd_port_cleanup(struct lldpd *cfg, struct lldpd_port *port, int all)
 #endif
 #ifdef ENABLE_DOT1
 	lldpd_vlan_cleanup(port);
+	lldpd_ppvid_cleanup(port);
+	lldpd_pi_cleanup(port);
 #endif
 	free(port->p_id);
 	free(port->p_descr);
@@ -558,6 +590,10 @@ lldpd_get_lsb_release() {
 static char *
 lldpd_get_os_release() {
 	static char release[1024];
+	char line[1024];
+	char *key, *val;
+	char *ptr1 = release;
+	char *ptr2 = release;
 
 	FILE *fp = fopen("/etc/os-release", "r");
 	if (!fp) {
@@ -565,9 +601,6 @@ lldpd_get_os_release() {
 		return NULL;
 	}
 
-	char line[1024];
-	char *key, *val;
-
 	while ((fgets(line, 1024, fp) != NULL)) {
 		key = strtok(line, "=");
 		val = strtok(NULL, "=");
@@ -580,8 +613,6 @@ lldpd_get_os_release() {
 	fclose(fp);
 
 	/* Remove trailing newline and all " in the string. */
-	char *ptr1 = release;
-	char *ptr2 = release;
 	while (*ptr1 != 0) {
 		if ((*ptr1 == '"') || (*ptr1 == '\n')) {
 			++ptr1;
@@ -891,18 +922,18 @@ lldpd_send_all(struct lldpd *cfg)
 static void
 lldpd_med(struct lldpd_chassis *chassis)
 {
-	free(chassis->c_med_hw);
-	free(chassis->c_med_fw);
-	free(chassis->c_med_sn);
-	free(chassis->c_med_manuf);
-	free(chassis->c_med_model);
-	free(chassis->c_med_asset);
-	chassis->c_med_hw = dmi_hw();
-	chassis->c_med_fw = dmi_fw();
-	chassis->c_med_sn = dmi_sn();
-	chassis->c_med_manuf = dmi_manuf();
-	chassis->c_med_model = dmi_model();
-	chassis->c_med_asset = dmi_asset();
+#if __i386__ || __amd64__
+	static short int once = 0;
+	if (!once) {
+		chassis->c_med_hw = dmi_hw();
+		chassis->c_med_fw = dmi_fw();
+		chassis->c_med_sn = dmi_sn();
+		chassis->c_med_manuf = dmi_manuf();
+		chassis->c_med_model = dmi_model();
+		chassis->c_med_asset = dmi_asset();
+		once = 1;
+	}
+#endif
 }
 #endif
 
@@ -930,9 +961,9 @@ lldpd_update_localchassis(struct lldpd *cfg)
 			fatal("failed to set full system description");
         } else {
 	        if (cfg->g_advertise_version) {
-		        if (asprintf(&LOCAL_CHASSIS(cfg)->c_descr, "%s %s %s %s",
+		        if (asprintf(&LOCAL_CHASSIS(cfg)->c_descr, "%s %s %s %s %s",
 			        cfg->g_lsb_release?cfg->g_lsb_release:"",
-				un.sysname, un.release, un.machine)
+				un.sysname, un.release, un.version, un.machine)
                                 == -1)
 			        fatal("failed to set full system description");
 	        } else {
@@ -1259,7 +1290,7 @@ lldpd_main(int argc, char *argv[])
 	cfg->g_delay = LLDPD_TX_DELAY;
 
 	/* Description */
-	if (!(cfg->g_advertise_version = advertise_version))
+	if (!(cfg->g_advertise_version = advertise_version) && lsb_release)
 		/* Remove the \n */
 		lsb_release[strlen(lsb_release) - 1] = '\0';
 	cfg->g_lsb_release = lsb_release;
